local threshold
local centralmonitor = {}
cm = centralmonitor

function cm.connection()
   return db.connect{api=db.SQLITE, name='callhome'}
end

function cm.reset(C)
   local C = cm.connection()
   C:execute{sql=[[DROP TABLE IF EXISTS callhome]]}
   C:execute{sql=[[CREATE TABLE callhome (
      server_guid TEXT NOT NULL,
      ts INTEGER,
      summary TEXT,
      PRIMARY KEY (server_guid))]]} 
   return {status='OK'}
end

function cm.mapRequest(T, R)
   T.server_guid = R.guid
   T.ts = os.ts.time()
   T.summary = R.summary
   return T
end

function cm.receiveData(R)   
   local C = cm.connection()
   local D = C:execute{
      sql=[[SELECT name FROM sqlite_master 
            WHERE type='table' AND name='callhome';]]
   }
   if #D == 0 then
      cm.reset(C)
   end
   local T = db.tables{vmd='callhome.vmd', name='Message'}
   local Data = json.parse{data=R.body}
   cm.mapRequest(T.callhome[1], Data)
   C:merge{data=T}
   return {status='OK'}
end

function node.S(N) return tostring(N) end

function cm.summary()
   local C = cm.connection()
 
   local CurrentTime = os.ts.time()
   local S = C:query{sql="SELECT server_guid, ts, summary FROM callhome"}
   local Guids = {}
   local j=1
   for i=1, #S do 
      local LastRefresh = os.ts.difftime(CurrentTime, tonumber(tostring(S[i].ts)))
      if (LastRefresh > threshold) then
         Guids[j] = S[i].server_guid:S()
      end
   end
   return Guids
end


function cm.detail(Data)
   trace(Data)
   local C = cm.connection()
   local D = C:query{sql=
      "SELECT ts, summary FROM callhome WHERE server_guid='" 
      .. Data.get_params.guid .. "'"}

   local Xml
   if #D > 0 then 
     Xml = xml.parse(D[1].summary:S())
   end

   local Details = cm.convertXmlToLua(Xml)
   local CurrentTime = os.ts.time()
   Details.server.LastRefresh = os.ts.difftime(CurrentTime, tonumber(tostring(D[1].ts)))

   return Details
end

function cm.timeago(Seconds)
   local mf = math.floor
   local FT = 'Less than a minute ago'
   if Seconds < 60 then
   -- leave FT alone
   elseif Seconds < 120 then
      FT = 'About a minute ago'
   elseif Seconds < 7200 then
      FT = mf(Seconds / 60) .. ' minutes ago'
   elseif Seconds < 2*86400 then
      FT = mf(Seconds / 3600) .. ' hours ago'
   else
      FT = mf(Seconds / 84600) .. ' days ago'
   end
   return FT
end


local ActionTable={
   ['/monitor/send']=cm.receiveData,
   ['/monitor/summary']=cm.summary,
   ['/monitor/reset']=cm.reset,
   ['/monitor/detail']=cm.detail,
}

function cm.convertXmlToLua(Xml)
   local T = {}
   T.server = {}
   T.channels = {}
   local S = T.server
   local Ch = T.channels
   local MT = Xml.IguanaStatus
   trace(MT)
   for i = 1, #MT do
      if MT[i]:nodeType() == 'attribute' then
         S[MT[i]:nodeName()] = MT[i]:nodeValue()
      else
         local C = MT[i]
         local Cvars = {}
         for j = 1, #C do
            Cvars[C[j]:nodeName()] = C[j]:nodeValue()     
         end
         Ch[C.Guid:S()] = Cvars
      end
      
   end
   return T
end

-- Find the method for the action.
function cm.serveRequest(Data, T)
   threshold = T
   local R = net.http.parseRequest{data=Data}
   trace(R.location)
   local Action = ActionTable[R.location]
   if (Action) then
      local Result = Action(R)
      Result = json.serialize{data=Result}
      net.http.respond{body=Result, entity_type='text/json'}   
      return
   end
   
   -- Default page
   net.http.respond{body=''}
end


function cm.checkForIssues(params)   
   local R = net.http.get{
      url=  params.url .. 'summary'
   }

   local Guids = json.parse{data=R}
   local issues = {server={}}
   local j=1
   
   for i = 1, #Guids do
      local N = net.http.get{
         url=        params.url .. 'detail', 
         parameters= {guid=Guids[i]}
      }
      issues.server[j] = json.parse{data=N} 
   end   
   return issues

end

function cm.buildEmail(issues, params)
   E = 'We haven\'t heard from the following server(s):\n\n'
   for i = 1, #issues.server do
      
      local server = issues.server[i].server
      local serverName = server.ServerName
      
      if serverName == '' then
         serverName = 'Unnamed Server'
      end
      
      
      E = E .. '********** ' .. serverName .. ' **********\n'
      E = E .. ' -- IP Address: ' .. server.IpAddress .. '\n'
      E = E .. ' -- Last seen: ' .. cm.timeago(server.LastRefresh) ..'\n'
      E = E .. ' -- Channels: ' .. server.NumberOfChannels
      E = E .. ' (' .. server.NumberOfRunningChannels .. ' running)\n'
      E = E .. ' -- Channels: '
      
      local channels = issues.server[i].channels
      for k,v in pairs(channels) do
         E = E .. v.Name .. ', '
      end
      E = E .. '\n\n'
   end 
   E = E .. 'Reset channels being monitored? - ' .. 
   params.url .. 'reset'
   return E
end

function cm.sendEmail(E, S, params)
   
   store.put('Status-Problems', S)
   local prefix = 'Probem: '
   if S == '0' then 
      prefix = 'Resolved: '
   end
   
   local Header = {
      Subject = prefix .. params.header.subject;
      From =    params.header.senderName .. 
      ' <' .. params.smtp.from .. '>'
   }   
   params.smtp.header = Header
   params.smtp.body = E
   
   local Results = net.smtp.send(params.smtp)
end

return centralmonitor
